QUnit.module \ES6

isFunction = -> typeof! it is \Function

{freeze} = Object
{iterator} = Symbol

eq = strictEqual

test 'WeakMap' !->
  ok isFunction(WeakMap), 'Is function'
  ok /native code/.test(WeakMap), 'looks like native'
  if \name of WeakMap => eq WeakMap.name, \WeakMap, 'name is "WeakMap"'
  eq WeakMap.length, 0, 'length is 0'
  ok \delete of WeakMap::, 'delete in WeakMap.prototype'
  ok \get    of WeakMap::, 'get in WeakMap.prototype'
  ok \has    of WeakMap::, 'has in WeakMap.prototype'
  ok \set    of WeakMap::, 'set in WeakMap.prototype'
  ok new WeakMap instanceof WeakMap, 'new WeakMap instanceof WeakMap'
  eq new WeakMap([[a = {}, b = {}]].values!).get(a), b, 'Init WeakMap from iterator #1'
  eq new WeakMap(new Map([[a = {}, b = {}]])).get(a), b, 'Init WeakMap from iterator #2'
  eq new WeakMap([[f = freeze({}), 42]]).get(f), 42, 'Support frozen objects'
  M = new WeakMap
  M.set freeze(f = {}), 42
  eq M.has(f), on
  eq M.get(f), 42
  M.delete f
  eq M.has(f), no
  eq M.get(f), void
  # return #throw
  done = no
  iter = [null, 1, 2]values!
  iter.return = -> done := on
  try => new WeakMap iter
  ok done, '.return #throw'
  ok !(\clear of WeakMap::), 'should not contains `.clear` method'
  # call @@iterator in Array with custom iterator
  a = []
  done = no
  a[iterator] = ->
    done := on
    [][iterator]call @
  new WeakMap a
  ok done
test 'WeakMap#delete' !->
  ok isFunction(WeakMap::delete), 'Is function'
  ok /native code/.test(WeakMap::delete), 'looks like native'
  M = new WeakMap!
    .set a = {}, 42
    .set b = {}, 21
  ok M.has(a) && M.has(b), 'WeakMap has values before .delete()'
  M.delete a
  ok !M.has(a) && M.has(b), 'WeakMap hasn`t value after .delete()'
test 'WeakMap#get' !->
  ok isFunction(WeakMap::get), 'Is function'
  ok /native code/.test(WeakMap::get), 'looks like native'
  M = new WeakMap!
  eq M.get({}), void, 'WeakMap .get() before .set() return undefined'
  M.set a = {}, 42
  eq M.get(a), 42, 'WeakMap .get() return value'
  M.delete a
  eq M.get(a), void, 'WeakMap .get() after .delete() return undefined'
test 'WeakMap#has' !->
  ok isFunction(WeakMap::has), 'Is function'
  ok /native code/.test(WeakMap::has), 'looks like native'
  M = new WeakMap!
  ok !M.has({}), 'WeakMap .has() before .set() return false'
  M.set a = {}, 42
  ok M.has(a), 'WeakMap .has() return true'
  M.delete a
  ok !M.has(a), 'WeakMap .has() after .delete() return false'
test 'WeakMap#set' !->
  ok isFunction(WeakMap::set), 'Is function'
  ok /native code/.test(WeakMap::set), 'looks like native'
  ok new WeakMap!set(a = {}, 42), 'WeakMap.prototype.set works with object as keys'
  ok (try new WeakMap!set(42, 42); no; catch => on), 'WeakMap.prototype.set throw with primitive keys'
test 'WeakMap#@@toStringTag' !->
  eq WeakMap::[Symbol?toStringTag], \WeakMap, 'WeakMap::@@toStringTag is `WeakMap`'